<?php
/**
 * Plugin Qcm (addon for plugin Adm_redacteurs)
 * Version  : 1.5 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Qcm_questions_model extends Model
{    
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    private $_data_DB = array(); // Données de la base de données
    private $_file_dir_path = QCM_QUESTIONS_PATH; // Chemin du répertoire contenant les fichiers des données des documents
    private $_file_prefix = QCM_QUESTIONS_FILE_PREFIX; // Préfixe du nom du fichier contenant les données du document
    private $_file_ext = INCEXT;

    private $_params = array(
        'id' => 0, // Id du document
        'qcm_id' => 0, // Id du qcm
        'type' => 'questions', // Type de question
        'title1' => '',  // Titre de la question (langue 1)
        'title2' => '',  // Titre de la question (langue 2)

        'content1' => '', // contenu de la question (langue 1)
        'content2' => '', // contenu de la question (langue 2)

        'answers' => array( // Réponses
            array('content1' => '', 'content2' => '', 'is_correct' => 0)
        ),
        'points' => array( // Nombre de points attribués aux questions
            array('number' => 1)
        ),
        
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */
    public function __construct()
	{

	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le nom des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_all_files()
	{
        $files = array();
        
        if (!empty($this -> _file_dir_path)) {
            $files = array_slice(scandir($this -> _file_dir_path), 2);
            unset($files[array_search('index.php', $files)]); // Suppression du fichier index.php
        }
        
        return $files;
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie les identifiants des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_files_ids()
	{
        $files_ids = array();
        $files = $this -> get_all_files();

        foreach($files as $key => $file) {
            $files_ids[] = str_replace(array($this -> _file_prefix, $this -> _file_ext), '', $file);
        }
        
        return $files_ids;
	}

	// --------------------------------------------------------------------

	/**
	 * Vérifie si le document existe ou non
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return boolean
	 */
    public function file_exists($doc_id = 0)
	{
        if (!empty($doc_id) && ctype_digit((string) $doc_id) && is_file($this -> _get_file_path($doc_id))) {            
            return TRUE;    
        }
        
        return FALSE;
	}

	// --------------------------------------------------------------------

	/**
	 * Lit et renvoie dans un tableau les données enregistrées dans le document identifié par la valeur de son id
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return array Tableau contenant les valeurs du document
	 */
    public function read($doc_id = 0)
    {
        $data = $this -> _params;        
        
        if ($this -> file_exists($doc_id)) {
            $data = array_merge($data, (array) unserialize(file_get_contents($this -> _get_file_path($doc_id))));    
        }
       
        return $data; 
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * Ecrit les données du document dans un fichier
     * 
     * @access public 
     * @param array Données du rédacteur ou du groupe rédacteur
	 * @return array
	 */    
    public function write($data)
    {
        settype($data, 'array');

        if (empty($data['id']) || ! ctype_digit((string) $data['id'])) {
            return FALSE;
        }
      
        $file_path = $this -> _get_file_path($data['id']);
        $data = $this -> _clean($data);
                
		return file_put_contents($file_path, serialize($data), LOCK_EX);
    }

	// --------------------------------------------------------------------
 
	/**
	 * Supprime les documents identifiés par leur id
     * 
     * @access public 
     * @param array|integer Identifiants des documents à supprimer
	 * @return boolean
	 */    
    public function delete_by_id($doc_ids)
    {
        settype($doc_ids, 'array');

        if (empty($this -> _file_dir_path) || empty($doc_ids)) {
            return FALSE;
        }
                
        foreach($doc_ids as $doc_id) {
            if ($this -> file_exists($doc_id)) {
                @unlink($this -> _get_file_path($doc_id));    
            }
        }
       
        return TRUE; 
    }
   
	// --------------------------------------------------------------------
 
	/**
	 * Retourne le chemin du fichier associé à l'id passé en paramètre
     * 
     * @access private 
     * @param integer Id du document
	 * @return string Chemin du fichier associé à l'id passé en paramètre
	 */    
    private function _get_file_path($doc_id)
    {
        return $this -> _file_dir_path.'/'.$this -> _file_prefix.$doc_id.$this -> _file_ext;    
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        $data_by_default = $this -> _params;
        $data = $this -> _parse_params_contents($data);
        $data['answers'] = $this -> _parse_params_answers_contents($data['answers']);
                
        return array_merge($data_by_default, array_intersect_key($data, $data_by_default));
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données de type "contenu"
     * 
     * @access private 
	 * @param array Données à nettoyer
	 * @return array
	 */
    private function _parse_params_contents($params)
    {
        $in = array(
    		'\\',
    		'="'.ADMR_SITE_URL
    	);

    	$out = array(
    		'&#92;',
    		'="'
    	);       
        
        foreach(array('content1', 'content2') as $key) {
            if (!empty($params[$key])) {
                $params[$key] = str_replace($in, $out, $params[$key]);
            }            
        }        

        return $params;
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données de type "contenu" des réponses
     * 
     * @access private 
	 * @param array Données à nettoyer
	 * @return array
	 */    
    private function _parse_params_answers_contents($answers)
    {
        foreach($answers as $key => $answer) {
            $answers[$key] = $this -> _parse_params_contents($answer);
        }

        return $answers;
    }

	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie les règles de validation des paramètres
     * 
     * @access public 
	 * @param array Tableau contenant les paramètres à valider
	 * @param array Tableau contenant les messages d'erreur
	 * @param array Tableau contenant les options de configuration
	 * @return array Tableau contenant les règles de validation et les messages d'erreur associés
	 */
    public function set_params_rules($params, $lang, $config)
    {
        return array_merge(
            Params_rules_helper :: set_params_author_rules($lang),            // Author
            Params_rules_helper :: set_params_email_rules($lang),             // Email
            Params_rules_helper :: set_params_status_rules($lang, (ADMR_IS_ADMIN) ? array('a', 'i', 'd') : array('a', 'i')), // Status
            Params_rules_helper :: set_params_group_rules($lang),             // Group privé
            Params_rules_helper :: set_params_title_rules($lang, $config),    // Titre du document

            array(
                'qcm_id' => array( // Identifiant du qcm
                    array(
                        'rules' => array(array('Var_helper', 'is_digit_no_zero')),
                        'msg' => htmlspecialchars($lang['form_errors_msg']['field_not_number_valid'], ENT_QUOTES, 'UTF-8')
                    ),
                ),

/*                'nb_questions' => array( // Nombre de questions du qcm
                    array(
                        'rules' => array(array('Var_helper', 'is_digit_no_zero')),
                        'msg' => htmlspecialchars($lang['form_errors_msg']['field_not_number_valid'], ENT_QUOTES, 'UTF-8')
                    ),
                ),*/
            )
        );
    }
     
	// --------------------------------------------------------------------

} // End of Class